/*
 * Copyright (c) 2002-2003 WIPRO TECHNOLOGIES 
 * Developed for LSB-PAM project by WIPRO TECHNOLOGIES
 */

/*
 * Project: LSB-PAM
 *
 * File: tset/LSB.pam/testcases/pam_fail_delay/pam_fail_delay.c
 * This file contains the test cases for testing the interface
 * pam_fail_delay.
 *
 * Author(s) : Rohit R <rohit.raveendran@wipro.com>, WIPRO TECHNOLOGIES
 *             Laavanya K R <laavanya.gopalan@wipro.com>, WIPRO TECHNOLOGIES
 *
 * Modifications:
 *
 */

#include <stdio.h>
#include <security/pam_appl.h>
#include <security/pam_misc.h>
#include <time.h>

#include <std.h>
#include <tet_api.h>

/*******************************************************************/

#define NUM_TESTS   3   /* To be updated when number of tests are modified */
#define ONE_SECOND  1000000    /* in micro seconds */

private void tc1();
private void tc2();
private void tc3();

private void prepare_tests();
private void clean_tests();

public  void (*tet_startup)() = prepare_tests;
public  void (*tet_cleanup)() = clean_tests;

private char *conf_file;
private char *user;
private char *passwd;
private char *input_passwd;

private char echo[256];

private pam_handle_t *pamh;
private struct pam_conv pamc;

private time_t start_time;
private time_t end_time;

public  struct tet_testlist tet_testlist[] = {
        { tc1, 1 },
        { tc2, 2 },
        { tc3, 3 },
        { NULL, 0 }
};

private int login_conv(int num_msg,
                       const struct pam_message **msg,
                       struct pam_response **response,
                       void *appdata_ptr);


private void delete_tests(char *msg);

/*******************************************************************/

private void
prepare_tests()
{
    int    retval;


    tet_infoline(" starting tests pam_fail_delay ");

    pamh = NULL;
    pamc.conv = &login_conv;
    pamc.appdata_ptr = NULL;

    conf_file = tet_getvar("LSB_PAM_CONF_FILE");

    if (conf_file == NULL) {
        delete_tests(" tetexec.cfg parameter LSB_PAM_CONF_FILE unset");
        return;
    }

    user = tet_getvar("LSB_PAM_USER");

    if (user == NULL) {
        delete_tests(" tetexec.cfg parameter LSB_PAM_USER unset");
        return;
    }

    passwd = tet_getvar("LSB_PAM_USER_PASSWD");

    if (passwd == NULL) {
        delete_tests(" tetexec.cfg parameter LSB_PAM_USER_PASSWD unset");
        return;
    }

    retval = pam_start(conf_file, user, &pamc, &pamh);

    if (retval != PAM_SUCCESS) {
        sprintf(echo," pam_start failed with %s",
                                pam_strerror(NULL, retval));
        delete_tests(echo);
        return;
    }

    return;
}


private void
clean_tests()
{
    if (pamh) {
        pam_end(pamh,0);
    }

    tet_infoline(" end of tests pam_fail_delay ");
    return;
}


/*
 *    tc1():
 *        This test case checks the interface pam_fail_delay.
 *        The expected return value is PAM_SUCCESS.
 */

private void
tc1()
{
    int    retval;
    int    delay_time;
    int    time_diff;
    int    lower_limit;
    int    upper_limit;

    delay_time = 2 * ONE_SECOND;

    /* Accepted delay time range is 25%. Considering + or - 2 more sec */
    lower_limit = ((delay_time - (delay_time * 0.25)) / ONE_SECOND) - 2;
    upper_limit = ((delay_time + (delay_time * 0.25)) / ONE_SECOND) + 2;

    retval = pam_fail_delay(pamh, delay_time);

    if (retval != PAM_SUCCESS) {
        sprintf(echo," pam_fail_delay returned %s when expected to succeed",
                                pam_strerror(pamh, retval));
        delete_tests(echo);

        tet_result(TET_FAIL);
        return;
    }				

    /* Explicitly making pam_authenticate to fail to test pam_fail_delay */
    input_passwd = "\0";

    time(&start_time);
    retval = pam_authenticate(pamh, PAM_DISALLOW_NULL_AUTHTOK);
    time(&end_time);

    time_diff = end_time - start_time;

    if ((time_diff < lower_limit) || (time_diff > upper_limit)) {
        tet_infoline(" delay time exceeds the 25% deviation ");
        tet_printf("got %d, expected between %d and %d\n",
                                time_diff, lower_limit, upper_limit);	
        tet_result(TET_FAIL);
        return;
    }

    if (retval == PAM_SUCCESS) {
        tet_infoline(" pam_authenticate returned PAM_SUCCESS when expected not to succeed.");

        tet_result(TET_FAIL);
        return;

    } else if (retval == PAM_MAXTRIES) {
        delete_tests(" PAM_MAXTRIES reached. Cannot continue...");

        tet_result(TET_FAIL);
        return;
    }

    /* Successful authenticate */
    input_passwd = passwd;
    retval = pam_authenticate(pamh, 0);

    if(retval != PAM_SUCCESS) {
        tet_printf(" pam_authenticate returned %s when expected to succeed", 
                                pam_strerror(pamh, retval));

        tet_result(TET_FAIL);
        return;
    }

    tet_result(TET_PASS);
    return;
}


/*
 *    tc2():
 *        This test case checks the interface pam_fail_delay when called
 *        multiple times. pam_fail_delay is expected to use the longest
 *        time specified by any of the calls.
 */

private void
tc2()
{
    int    retval;
    int    delay_time[] = { 2, 5, 20, 3, 10 }; /* change max_delay also */
    int    index;
    int    time_diff;
    int    max_delay;
    int    lower_limit;
    int    upper_limit;
    int    count;

    /* This should be the max value in the array delay_time[] */
    max_delay = 20 * ONE_SECOND;

    /* accepted delay time range is 25%. Considering + or - 2 more sec */
    lower_limit = ((max_delay - (max_delay * 0.25)) / ONE_SECOND) - 2;
    upper_limit = ((max_delay + (max_delay * 0.25)) / ONE_SECOND) + 2;

    count = sizeof(delay_time) / sizeof(int);

    for (index = 0; index < count; index++) {
        retval = pam_fail_delay(pamh, (delay_time[index] * ONE_SECOND));

        if (retval != PAM_SUCCESS) {
            sprintf(echo," pam_fail_delay returned %s when expected to succeed",
                                pam_strerror(pamh, retval));
            delete_tests(echo);

            tet_result(TET_FAIL);
            return;
        }
    }

    /* Explicitly making pam_authenticate to fail to test pam_fail_delay */
    input_passwd = "\0";

    time(&start_time);
    retval = pam_authenticate(pamh, PAM_DISALLOW_NULL_AUTHTOK);
    time(&end_time);

    time_diff = end_time - start_time;

    if ((time_diff < lower_limit) || (time_diff > upper_limit)) {
        tet_infoline(" delay time exceeds the 25% deviation");
        tet_printf("got %d, expected between %d and %d\n",
                                time_diff, lower_limit, upper_limit);	
        tet_result(TET_FAIL);
        return;
    }

    if (retval == PAM_SUCCESS) {
        tet_infoline(" pam_authenticate returned PAM_SUCCESS when not expected to succeed.");

        tet_result(TET_FAIL);
        return;

    } else if (retval == PAM_MAXTRIES) {
        delete_tests(" PAM_MAXTRIES reached. Cannot continue...");
        tet_result(TET_FAIL);
        return;
    }

    /* Successful authenticate */
    input_passwd = passwd;
    retval = pam_authenticate(pamh, 0);

    if(retval != PAM_SUCCESS) {
        tet_printf(" pam_authenticate returned %s when expected to succeed.",
                                pam_strerror(pamh, retval));

        tet_result(TET_FAIL);
        return;
    }

    tet_result(TET_PASS);
    return;
}


/*
 *    tc3():
 *        This test case checks the interface pam_fail_delay with NULL
 *        as the pam handle. The return value should not be PAM_SUCCESS.
 */

private void
tc3()
{
    int    retval;


    retval = pam_fail_delay(NULL, ONE_SECOND);

    if (retval == PAM_SUCCESS) {
        tet_infoline(" pam_fail_delay returned PAM_SUCCESS with NULL pamh");
        tet_infoline(" when not expected to succeed.");

        tet_result(TET_FAIL);
        return;
    }

    tet_result(TET_PASS);
    return;
}


/*
 *   login_conv():
 *        This routine is called to get the username and password
 *        required for authenticating the user. Currently the
 *        username and password are got from the configuration files
 *        to avoid user intervention.
 */

private int
login_conv(int num_msg,
           const struct pam_message **m,
           struct pam_response **r,
           void *appdata_ptr)
{
    struct  pam_response *reply;
    int     count;
    char    buf[PAM_MAX_RESP_SIZE];
    char    *temp;

    reply = (struct pam_response *)
                calloc(num_msg, sizeof(struct pam_response));

    if (reply == NULL) {
        return PAM_BUF_ERR;
    }
	
    for (count = 0; count < num_msg; count++) {
        reply[count].resp_retcode = 0;
        reply[count].resp = NULL;

        switch (m[count]->msg_style) {

            case PAM_PROMPT_ECHO_OFF:

                reply[count].resp = strdup(input_passwd);
                break;

            case PAM_PROMPT_ECHO_ON:

                reply[count].resp = strdup(user);
                break;

            default:
                break;
        }
    }

    *r = reply;
    return PAM_SUCCESS;
}


/*
 *  delete_tests():
 *       This routine is called to stop execution of the test cases
 *       in case of initialization errors.
 */

private void
delete_tests(char *msg)
{
    int   count;

    for (count = 1; count <= NUM_TESTS; count++) {
        tet_delete(count, msg);
    }

    return;
}
