/*
 * Copyright (c) 2002-2003 WIPRO TECHNOLOGIES 
 * Developed for LSB-PAM project by WIPRO TECHNOLOGIES
 */

/*
 * Project: LSB-PAM
 *
 * File: tset/LSB.pam/testcases/pam_get_item/pam_get_item.c
 * This file contains the test cases for testing the interface
 * pam_get_item.
 *
 * Author(s) : Rohit R <rohit.raveendran@wipro.com>, WIPRO TECHNOLOGIES
 *             Laavanya K R <laavanya.gopalan@wipro.com>, WIPRO TECHNOLOGIES
 *
 * Modifications:
 *
 */

#include <stdio.h>
#include <security/pam_appl.h>
#include <security/pam_misc.h>

#include <std.h>
#include <tet_api.h>

/*******************************************************************/

#define NUM_TESTS   4 /* To be updated when number of tests are modified */

private void tc1();
private void tc2();
private void tc3();
private void tc4();

private void prepare_tests();
private void clean_tests();

public  void (*tet_startup)() = prepare_tests;
public  void (*tet_cleanup)() = clean_tests;

private char *conf_file;
private char *user;

private char echo[256];

private pam_handle_t *pamh;
private struct pam_conv pamc;

struct mapping {
        int   type;
        char  *string;
};

private struct mapping item_types[] = {
        { PAM_SERVICE, "PAM_SERVICE" },
        { PAM_USER, "PAM_USER" },
        { PAM_TTY, "PAM_TTY" },
        { PAM_RHOST, "PAM_RHOST" },
        { PAM_CONV, "PAM_CONV" },
        { PAM_RUSER, "PAM_RUSER" },
        { PAM_USER_PROMPT, "PAM_USER_PROMPT" }
};

public struct tet_testlist tet_testlist[] = {
        { tc1, 1 },
        { tc2, 2 },
        { tc3, 3 },
        { tc4, 4 },
        { NULL, 0 }
};

private void delete_tests(char *msg);

/*******************************************************************/

private void
prepare_tests()
{
    int    retval;

    tet_infoline(" starting tests pam_get_item");

    pamh = NULL;
    pamc.conv = &misc_conv;
    pamc.appdata_ptr = NULL;

    conf_file = tet_getvar("LSB_PAM_CONF_FILE");

    if (conf_file == NULL) {
        delete_tests(" tetexec.cfg parameter LSB_PAM_CONF_FILE unset");
        return;
    }

    user = tet_getvar("LSB_PAM_USER");

    if (user == NULL) {
        delete_tests(" tetexec.cfg parameter LSB_PAM_USER unset");
        return;
    }

    retval = pam_start(conf_file, user, &pamc, &pamh);

    if (retval != PAM_SUCCESS) {
        sprintf(echo," pam_start returned %s when expected to succeed.",
                                pam_strerror(NULL, retval));
        delete_tests(echo);
        return;
    }

    return;
}


private void
clean_tests()
{
    if (pamh) {
        pam_end(pamh,0);
    }

    tet_infoline(" end of tests pam_get_item");
    return;
}


/*
 *    tc1() :
 *         This test case checks the interface pam_get_item for
 *         valid item types. Expected return value is PAM_SUCCESS
 */

private void
tc1()
{
    int    retval;
    int    num = 0;
    int    count = 0;
    char   *value;


    num = sizeof(item_types) / sizeof(struct mapping);

    for (count = 0; count < num; count++) {

        retval = pam_get_item(pamh, item_types[count].type,
                                        (const void **)&value);

        if (retval != PAM_SUCCESS) {
            tet_printf(" pam_get_item failed to get value for %s.Returned %s\n",
                         item_types[count].string, pam_strerror(pamh, retval));
            tet_result(TET_FAIL);
            return;
        }
    }

    tet_result(TET_PASS);
    return;
}


/*
 *    tc2() :
 *        This test case tests the interface pam_get_item by passing
 *        a bad item type. Expected return type is PAM_BAD_ITEM
 */

private void
tc2()
{
    int    retval;
    char   *value;


    retval = pam_get_item(pamh, PAM_SUCCESS, (const void **) &value);

    if (retval != PAM_BAD_ITEM) {
        tet_printf(" pam_get_item returned %s when expecting PAM_BAD_ITEM\n",
                                pam_strerror(pamh, retval));

        tet_result(TET_FAIL);
        return;
    }

    tet_result(TET_PASS);
    return;
}


/*
 *    tc3() :
 *        This test case checks the interface pam_get_item by passing a
 *        NULL pointer for the return address. Expected return value is 
 *        PAM_PERM_DENIED
 */

private void
tc3()
{
    int    retval;
    int    num = 0;
    int    count = 0;


    num = sizeof(item_types) / sizeof(struct mapping);

    for (count = 0; count < num; count++) {

        retval = pam_get_item(pamh, item_types[count].type, NULL);

        if (retval != PAM_PERM_DENIED) {
            tet_printf(" pam_get_item returned %s for %s ",
                        pam_strerror(pamh, retval), item_types[count].string);
            tet_infoline(" Expected to return PAM_PERM_DENIED");

            tet_result(TET_FAIL);
            return;
        }
    }

    tet_result(TET_PASS);
    return;
}


/*
 *    tc4():
 *        This test case checks the interface pam_get_item with
 *        a NULL pam handle. The return value should not be PAM_SUCCESS.
 */
private void
tc4()
{
    int    retval;
    char   *value;
    int    count = 1;

    retval = pam_get_item(NULL, item_types[count].type,
                                        (const void **)&value);

    if (retval == PAM_SUCCESS) {
        tet_infoline(" pam_get_item returned PAM_SUCCESS when not expected to");
        tet_result(TET_FAIL);
        return;
    }

    tet_result(TET_PASS);
    return;
}



/*
 *  delete_tests():
 *       This routine is called to stop execution of the test cases
 *       in case of initialization errors.
 */

private void
delete_tests(char *msg)
{
    int    count;

    for (count = 1; count <= NUM_TESTS; count++) {
        tet_delete(count, msg);
    }

    return;
}
