/*
 * Copyright (c) 2002-2003 WIPRO TECHNOLOGIES 
 * Developed for LSB-PAM project by WIPRO TECHNOLOGIES
 */

/*
 * Project: LSB-PAM
 *
 * File: tset/LSB.pam/testcases/pam_set_item/pam_set_item.c
 * This file contains the test cases for testing the interface
 * pam_set_item.
 *
 * Author(s) : Rohit R <rohit.raveendran@wipro.com>, WIPRO TECHNOLOGIES
 *             Laavanya K R <laavanya.gopalan@wipro.com>, WIPRO TECHNOLOGIES
 *
 * Modifications:
 *
 */

#include <stdio.h>
#include <security/pam_appl.h>
#include <security/pam_misc.h>

#include <std.h>
#include <tet_api.h>

/*******************************************************************/

#define NUM_TESTS   4 /* To be updated when number of tests are modified */

private void tc1();
private void tc2();
private void tc3();
private void tc4();

private void prepare_tests();
private void clean_tests();

public  void (*tet_startup)() = prepare_tests;
public  void (*tet_cleanup)() = clean_tests;

private char *conf_file;
private char *user;

private char echo[256];

private pam_handle_t *pamh;
private struct pam_conv pamc;
private struct pam_conv pamc_new;

struct mapping {
        int type;            /* Item type */
        char *string;        /* Description of item type */
        char *new_value;     /* New value for the item type */
};

private struct mapping item_types[] = {
        { PAM_SERVICE, "PAM_SERVICE", "login" },
        { PAM_USER, "PAM_USER", "guest" },
        { PAM_TTY, "PAM_TTY", "tty0" },
        { PAM_RHOST, "PAM_RHOST", "machine1" },
        { PAM_RUSER, "PAM_RUSER", "guest" },
        { PAM_USER_PROMPT, "PAM_USER_PROMPT", "Username : " }
};


public struct tet_testlist tet_testlist[] = {
        { tc1, 1 },
        { tc2, 2 },
        { tc3, 3 },
        { tc4, 4 },
        { NULL, 0 }
};

private void delete_tests(char *msg);

/*******************************************************************/

private void
prepare_tests()
{
    int    retval;

    tet_infoline(" starting tests pam_set_item");

    pamh = NULL;
    pamc.conv = &misc_conv;
    pamc.appdata_ptr = NULL;

    conf_file = tet_getvar("LSB_PAM_CONF_FILE");

    if (conf_file == NULL) {
        delete_tests(" tetexec.cfg parameter LAB_PAM_CONF_FILE unset");
        return;
    }

    user = tet_getvar("LSB_PAM_USER");

    if (user == NULL) {
        delete_tests(" tetexec.cfg parameter LAB_PAM_USER unset");
        return;
    }

    retval = pam_start(conf_file, user, &pamc, &pamh);

    if (retval != PAM_SUCCESS) {
        sprintf(echo," pam_start returned %s when expected to succeed",
                                pam_strerror(NULL, retval));
        delete_tests(echo);
        return;
    }

    return;
}


private void
clean_tests()
{
    if (pamh) {
        pam_end(pamh,0);
    }

    tet_infoline(" end of tests pam_set_item");
    return;
}


/*
 *    tc1() :
 *        This test case checks the interface pam_set_item by setting
 *        new values for the different item types. Expected return 
 *        value is PAM_SUCCESS
 *        After the test the old values are restored.
 */

private void
tc1()
{
    int    retval;
    int    count = 0;
    int    num = 0;
    char   *orig_value = NULL;
    char   *value;        /* Should not free/overwrite this pointer */


    /* Get the number of test items */
    num = sizeof(item_types) / sizeof(struct mapping);

    for (count = 0; count < num; count++) {

        /* Get the current value of the test item */
        retval = pam_get_item(pamh, item_types[count].type,
                                        (const void **)&value);

        if (retval != PAM_SUCCESS) {
            tet_printf(" pam_get_item failed to get current %s",
                                item_types[count].string);
            tet_result(TET_FAIL);
            return;
        }

        if (value == NULL) {
            continue;
        }

        if (orig_value) {
            free(orig_value);
            orig_value = NULL;
        }

        /* Save the orig value of item */
        orig_value = strdup(value);
        if (orig_value == NULL) {
            tet_infoline(" strdup failed ");
            continue;
        }

        /* Set the new value for the test item */
        retval = pam_set_item(pamh, item_types[count].type,
                                    item_types[count].new_value);

        if (retval != PAM_SUCCESS) {
            tet_printf(" pam_set_item failed to set %s to %s",
                    item_types[count].string, item_types[count].new_value);

            tet_printf(" It returned %s when expected to succeed.",
                                pam_strerror(pamh, retval));
            tet_result(TET_FAIL);

            if (orig_value) {
                free(orig_value);
            }
            return;
        }

        /* Restore the old value of the test item */
        pam_set_item(pamh, item_types[count].type, orig_value);
        free(orig_value);
        orig_value = NULL;

    }

    /* Get the current conversation structure */
    retval = pam_get_item(pamh, PAM_CONV, (const void **) &value);

    if (retval != PAM_SUCCESS) {
        tet_infoline(" pam_get_item failed to get current PAM_CONV type");
        tet_infoline(" It was expected to succeed");

        tet_result(TET_FAIL);
        return;
    }

    if (orig_value) {
        free(orig_value);
        orig_value = NULL;
    }

    /* Save the current value */
    if (value) {
        orig_value = strdup(value);
    }

    if (orig_value == NULL) {
        tet_infoline(" strdup failed ");
        tet_result(TET_FAIL);
        return;
    }

    /* Replace the conversation structure with new one */
    retval = pam_set_item(pamh, PAM_CONV, &pamc_new);

    if (retval != PAM_SUCCESS ) {
        tet_infoline(" pam_set_item failed to set PAM_CONV to new value");

        tet_result(TET_FAIL);
        free(orig_value);
        return;
    }

    /* Restore the old conversation structure */
    pam_set_item(pamh, PAM_CONV, orig_value);
    tet_result(TET_PASS);

    free(orig_value);
    return;
}


/*
 *    tc2() :
 *        This test checks the pam_set_item interface by passing
 *        NULL for the item PAM_CONV. Expected return value is
 *        PAM_PERM_DENIED
 */

private void
tc2()
{
    int    retval;
    char   *conv;        /* Should not free/overwrite this pointer */


    retval = pam_get_item(pamh, PAM_CONV, (const void **) &conv);

    if (retval != PAM_SUCCESS) {
        tet_infoline(" pam_get_item: failed to get current conv structure");
        tet_result(TET_FAIL);
        return;
    }

    /* Trying to set NULL for conversation structure */
    retval = pam_set_item(pamh, PAM_CONV, NULL);

    if (retval != PAM_PERM_DENIED ) {
        tet_infoline(" Set the conversation structure to NULL");
        tet_printf(" pam_set_item returned %s instead of PAM_PERM_DENIED",
                                pam_strerror(pamh, retval));

        tet_result(TET_FAIL);
        return;
    }

    pam_set_item(pamh, PAM_CONV, conv);
    tet_result(TET_PASS);
    return;
}


/*
 *    tc3() :
 *        This test case checks the pam_set_item interface by trying to
 *        update a non existing item type. Expected return value is
 *        PAM_BAD_ITEM
 */

private void
tc3()
{
    int    retval;

    
    retval = pam_set_item(pamh, PAM_SUCCESS, "Username: ");

    if (retval != PAM_BAD_ITEM) {
        tet_printf(" pam_set_item returned %s when expecting PAM_BAD_ITEM",
                                pam_strerror(pamh, retval));

        tet_result(TET_FAIL);
        return;
    }

    tet_result(TET_PASS);
    return;
}


/*
 *    tc4():
 *        This test case checks the interface pam_set_item with
 *        a NULL pam handle. The return value should not be PAM_SUCCESS.
 */

private void
tc4()
{
    int    retval;
    char   *conv;        /* Should not free/overwrite this pointer */


    retval = pam_get_item(pamh, PAM_CONV, (const void **) &conv);

    if (retval != PAM_SUCCESS) {
        tet_infoline(" pam_get_item : failed to get current conv structure");
        tet_result(TET_FAIL);
        return;
    }

    retval = pam_set_item(NULL, PAM_CONV, &pamc_new);

    if (retval == PAM_SUCCESS ) {
        tet_infoline(" pam_set_item returned PAM_SUCCESS for NULL pamh");
        tet_infoline(" It was expected not to be successful");

        tet_result(TET_FAIL);
        pam_set_item(pamh, PAM_CONV, &pamc);
        return;
    }

    tet_result(TET_PASS);
    return;
}




/*
 *  delete_tests():
 *       This routine is called to stop execution of the test cases
 *       in case of initialization errors.
 */

private void
delete_tests(char *msg)
{
    int    count;

    for (count = 1; count <= NUM_TESTS; count++) {
        tet_delete(count, msg);
    }

    return;
}
